' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

FUNCTION PBMAIN () AS LONG

   LOCAL nWidth, nHeight, running, frames, x, y AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL hr, i AS LONG
   LOCAL masking AS LONG         ' Masking on/off
   LOCAL mp AS LONG              ' M pressed?
   LOCAL sp AS LONG              ' Space pressed?
   LOCAL scene AS LONG           ' Which scene to draw
   LOCAL roll AS SINGLE          ' Rolling texture

   DIM   TextureHandles(4) AS DWORD
   DIM   TextureWidth(4) AS LONG
   DIM   TextureHeight(4) AS LONG
   DIM   strTextureData(4) AS STRING

   masking = %TRUE

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load textures from disk
   hr = GdiPlusLoadTexture("logo.bmp", TextureWidth(0), TextureHeight(0), strTextureData(0), %TRUE)
   hr = GdiPlusLoadTexture("mask1.bmp", TextureWidth(1), TextureHeight(1), strTextureData(1), %TRUE)
   hr = GdiPlusLoadTexture("image1.bmp", TextureWidth(2), TextureHeight(2), strTextureData(2), %TRUE)
   hr = GdiPlusLoadTexture("mask2.bmp", TextureWidth(3), TextureHeight(3), strTextureData(3), %TRUE)
   hr = GdiPlusLoadTexture("image2.bmp", TextureWidth(4), TextureHeight(4), strTextureData(4), %TRUE)

   ' Create two textures
   glGenTextures 5, TextureHandles(0)
   FOR i = 0 TO 4
      glBindTexture %GL_TEXTURE_2D, TextureHandles(i)
      glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
      glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
      glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth(i), TextureHeight(i), 0, _
                   %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData(i))
   NEXT

   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Enables clearing of the depth buffer
   glClearDepth 1.0!
   ' Enable depth testing
   glEnable %GL_DEPTH_TEST
   ' Enables smooth color shading
   glShadeModel %GL_SMOOTH
   ' Enable 2D texture mapping
   glEnable %GL_TEXTURE_2D

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 20 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

      ' Select and setup the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      ' Move into the screen 5 units
      glTranslatef 0.0!, 0.0!, -2.0!

      ' Select the logo texture
      glBindTexture %GL_TEXTURE_2D, TextureHandles(0)
      ' Start drawing a textured quad
      glBegin %GL_QUADS
         glTexCoord2f 0.0!, - roll + 0.0! : glVertex3f -1.1!, -1.1!,  0.0!  ' Bottom left
         glTexCoord2f 3.0!, - roll + 0.0! : glVertex3f  1.1!, -1.1!,  0.0!  ' Bottom right
         glTexCoord2f 3.0!, - roll + 3.0! : glVertex3f  1.1!,  1.1!,  0.0!  ' Top right
         glTexCoord2f 0.0!, - roll + 3.0! : glVertex3f -1.1!,  1.1!,  0.0!  ' Top left
      glEnd

      glEnable %GL_BLEND                           ' Enable blending
      glDisable %GL_DEPTH_TEST                     ' Disable depth testing

      IF masking THEN                              ' Is masking enabled?
         glBlendFunc %GL_DST_COLOR, %GL_ZERO       ' Blend screen color with zero (black)
      END IF

      IF scene THEN                                ' Are we drawing the second scene?
         glTranslatef 0.0!, 0.0!, -1.0!            ' Translate into the screen one unit
         glRotatef roll * 360, 0.0!, 0.0!, 1.0!    ' Rotate on the z axis 360 degrees
         IF masking THEN                           ' Is masking on?
            glBindTexture %GL_TEXTURE_2D, TextureHandles(3)   ' Select the second mask texture
            glBegin %GL_QUADS                      ' Start drawing a textured quad
               glTexCoord2f 0.0!, 0.0! : glVertex3f -1.1!, -1.1!, 0.0!   ' Bottom left
               glTexCoord2f 1.0!, 0.0! : glVertex3f  1.1!, -1.1!, 0.0!   ' Bottom right
               glTexCoord2f 1.0!, 1.0! : glVertex3f  1.1!,  1.1!, 0.0!   ' Top right
               glTexCoord2f 0.0!, 1.0! : glVertex3f -1.1!,  1.1!, 0.0!   ' Top left
            glEnd                                  ' Done Drawing The Quad
         END IF

         glBlendFunc %GL_ONE, %GL_ONE                      ' Copy image 2 color to the screen
         glBindTexture %GL_TEXTURE_2D, TextureHandles(4)   ' Select the second image texture
         glBegin %GL_QUADS                                 ' Start drawing a textured quad
            glTexCoord2f 0.0!, 0.0! : glVertex3f -1.1!, -1.1!,  0.0!     ' Bottom left
            glTexCoord2f 1.0!, 0.0! : glVertex3f  1.1!, -1.1!,  0.0!     ' Bottom right
            glTexCoord2f 1.0!, 1.0! : glVertex3f  1.1!,  1.1!,  0.0!     ' Top right
            glTexCoord2f 0.0!, 1.0! : glVertex3f -1.1!,  1.1!,  0.0!     ' Top left
         glEnd                                             ' Done Drawing The Quad
      ELSE
         IF masking THEN                                     ' Is masking on?
            glBindTexture %GL_TEXTURE_2D, TextureHandles(1)  ' Select the first mask texture
            glBegin %GL_QUADS                                ' Start drawing a textured quad
               glTexCoord2f roll + 0.0!, 0.0! : glVertex3f -1.1!, -1.1!, 0.0!  ' Bottom left
               glTexCoord2f roll + 4.0!, 0.0! : glVertex3f  1.1!, -1.1!, 0.0!  ' Bottom right
               glTexCoord2f roll + 4.0!, 4.0! : glVertex3f  1.1!,  1.1!, 0.0!  ' Top right
               glTexCoord2f roll + 0.0!, 4.0! : glVertex3f -1.1!,  1.1!, 0.0!  ' Top left
            glEnd                                            ' Done drawing the quad
         END IF

         glBlendFunc %GL_ONE, %GL_ONE                        ' Copy image 1 color to the screen
         glBindTexture %GL_TEXTURE_2D, TexTureHandles(2)     ' Select the first image texture
         glBegin %GL_QUADS                                   ' Start drawing a textured quad
            glTexCoord2f roll + 0.0!, 0.0! : glVertex3f -1.1!, -1.1!, 0.0!  ' Bottom left
            glTexCoord2f roll + 4.0!, 0.0! : glVertex3f  1.1!, -1.1!, 0.0!  ' Bottom right
            glTexCoord2f roll + 4.0!, 4.0! : glVertex3f  1.1!,  1.1!, 0.0!  ' Top right
            glTexCoord2f roll + 0.0!, 4.0! : glVertex3f -1.1!,  1.1!, 0.0!  ' Top left
         glEnd                                               ' Done drawing the quad
      END IF

      glEnable %GL_DEPTH_TEST                    ' Enable depth testing
      glDisable %GL_BLEND                        ' Disable blending

      roll = roll + 0.002!                       ' Increase our texture roll bariable
      IF roll > 1.0! THEN                        ' Is roll greater than one
         roll = roll - 1.0!                      ' Subtract 1 from roll
      END IF

      ' Swap buffers
      glfwSwapBuffers

      ' Space
      IF glfwGetKey(32) = %GLFW_PRESS AND ISFALSE sp THEN
         sp = %TRUE
         scene = NOT scene
      END IF
      IF glfwGetKey(32) <> %GLFW_PRESS THEN sp = %FALSE

      ' M key
      IF glfwGetKey(77) = %GLFW_PRESS AND ISFALSE mp THEN
         mp = %TRUE
         IF ISTRUE masking THEN masking = %FALSE ELSE masking = %TRUE
      END IF
      IF glfwGetKey(77) <> %GLFW_PRESS THEN mp = %FALSE

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the textures
   glDeleteTextures(5, TextureHandles(0))

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
